library(TSA)

# Simulate a time series using the time domain
# AR-1 Process
phi1 <- 0.8
y <- w <- rnorm(1000)
for (t in 2:1000) y[t] <- phi1 * y[t-1] + w[t]
par(mfrow= c(3,1))
plot(y, type="l")
acf(y)
pacf(y)

 # Simulate a time series using the frequency domain
c = 2*cos(2*pi*(1:500)/50 + .6*pi)
w = rnorm(500,0,1)
par(mfrow=c(3,1), mar=c(3,2,2,1), cex.main=1.5)
plot.ts(c, main=expression(x[t]==2*cos(2*pi*t/50+.6*pi)))
plot.ts(c+w, main=expression(x[t]==2*cos(2*pi*t/50+.6*pi)+N(0,1)))
plot.ts(c+5*w, main=expression(x[t]==2*cos(2*pi*t/50+.6*pi)+N(0,25)))
# Notice that we are adding progressively more white noise to the underlying process.
# However, the underlying signal is still there.

# Example 4.1 from Shumway Shows how a time series can be composed of various
# Fundamental Frequencies
 t = 1:100
 x1 = 2*cos(2*pi*t*6/100) + 3*sin(2*pi*t*6/100)
 x2 = 4*cos(2*pi*t*10/100) + 5*sin(2*pi*t*10/100)
 x3 = 6*cos(2*pi*t*40/100) + 7*sin(2*pi*t*40/100)
 x = x1 + x2 + x3
 par(mfrow=c(2,2))
 plot.ts(x1, ylim=c(-10,10), main = expression(omega==6/100~~~A^2==13))
 plot.ts(x2, ylim=c(-10,10), main = expression(omega==10/100~~~A^2==41))
 plot.ts(x3, ylim=c(-10,10), main = expression(omega==40/100~~~A^2==85))
 plot.ts(x, ylim=c(-16,16),main="sum")

# Plot of Cosine Curves with n=96 and Two Frequencies and Phases
win.graph(width=4.875,height=2.5,pointsize=8)
t=1:96 # n=96
cos1=cos(2*pi*t*4/96)
cos2=cos(2*pi*(t*14/96+.3))
plot(t,cos1, type='o', ylab='Cosines')
lines(t,cos2,lty='dotted',type='o',pch=4)

# Plot of Linear Combination of Two Cosine Curves
win.graph(width=4.875,height=2.5,pointsize=8)
t=1:96 # n=96
cos1=cos(2*pi*t*4/96)
cos2=cos(2*pi*(t*14/96+.3))
y=2*cos1+3*cos2
plot(t,y,type='o',ylab=expression(y[t]))

# Compute the periodogram of the preceding using the base package fft procedure
P = abs(2*fft(y)/100)^2
f = 0:50/100
par(mfrow=c(1,1))
plot(f, P[1:51], type="o", xlab="frequency", ylab="periodogram")
abline(h=0)
axis(1,at=c(0.04167,.14583))

# You can also get the periodogram using the base package spectrum command with no smoothing.
spectrum(y, log=c("no"))
abline(h=0)
axis(1,at=c(0.04167,.14583))

# Compute the periodogram of the preceding using TSA
periodogram(y)
abline(h=0)
axis(1,at=c(0.04167,.14583))

# Spectrum of White Noise with Different Smoothing
layout(1:3)
set.seed(1)
x<- rnorm(2048)
spectrum(x, log=c("no"))
spectrum(x, span=15, log=c("no"))
spectrum(x, span=65, log=c("no"))


# Does the periodogram work just as well when we do not know where or even if there are cosines in the series?
# What if a series contains additional noise?
# To illustrate, we generate a time series using randomness to select the frequencies, amplitudes, and phases.
# The two frequencies are randomly chosen without replacement from among 1/96, 2/96,...47/96. Additionally,
# a white noise series with zero mean and standard deviation 1 is chosen independently and added on.
win.graph(width=4.875,height=2.5,pointsize=8)
t=1:96 # n=96
set.seed(134)
integer=sample(48,2) # randomly sample 2 integers from 1:48 without replacement
freq1=integer[1]/96; freq2=integer[2]/96
A1=rnorm(1,0,2); B1=rnorm(1,0,2)  # sample normal "amplitudes"
A2=rnorm(1,0,3); B2=rnorm(1,0,3)
w=2*pi*t
y=A1*cos(w*freq1)+B1*sin(w*freq1)+A2*cos(w*freq2)+B2*sin(w*freq2)+rnorm(96,0,1)
plot(t,y,type='o',ylab=expression(y[t]))

# Compute the periodogram of this series
periodogram(y) ; abline(h=0)

# This periodogram shows that the series contains two cosine-sine pairs at frequencies of about 0.11 and 0.32,
# and that the higher frequency component is much stronger. There are also some small spikes resulting from
# the white noise.

# Data from Whittaker and Robinson on brightness (amplitude) of a particular star at midnight on 600 consecutive
# nights
par(mfrow=c(2,1))
data(star)
plot(star,xlab='Day',ylab='Brightness')
periodogram(star,ylab='Variable Star Periodogram');  abline(h=0)

# Look at the actual numerical values of the peaks
numbers = periodogram(star)
numbers$spec

# Inspecting the numerical values of the peaks, we see two very prominent peaks. The larger peak occurs at f=21/600.
# This frequency corresponds with a period of 600/21, or about 28.57 days. The secondary peak occurs at f=25/600. This frequency
# corresponds with a period of 600/25, or about 24 days. There appear to be periodicities in brightness corresponding
# with about 29 and 24 days.

# Aliasing illustrated
# Here we plot two cosine waves, one with f=1/4 and a second with f=3/4
win.graph(width=4.875, height=2.5,pointsize=8)
t=seq(0,8,by=.05)
plot(t,cos(2*pi*t/4),axes=F,type='l',ylab=expression(Y[t]),
 xlab='Discrete Time t')
axis(1,at=c(1,2,3,4,5,6,7))
axis(1); axis(2); box()
lines(t,cos(2*pi*t*3/4),lty='dashed',type='l')
abline(h=0)
points(x=c(0:8),y=cos(2*pi*c(0:8)/4),pch=19)

# Notice that the maxima and crossings correspond. In this case, we say that the two series alias one another.

# The theoretical spectral density for various ARMA models.
# MA1 spectral density
win.graph(width=4.875, height=2.5,pointsize=8)
theta=.9 # Reset theta for other MA(1) plots
ARMAspec(model=list(ma=-theta))
# R uses the plus convention in the MA specification.

# MA(2) models
win.graph(width=4.875, height=2.5,pointsize=8)
theta1=1; theta2=-0.6 # Reset values of theta1 & theta2 for other MA(2) models
ARMAspec(model=list(ma=-c(theta1,theta2)))

# AR(1)
win.graph(width=4.875, height=2.5,pointsize=8)
phi=0.9 # Reset value of phi for other AR(1) models
ARMAspec(model=list(ar=phi))

# AR(2)
win.graph(width=4.875, height=2.5,pointsize=8)
phi1=1.5; phi2=-.75  # Reset values of phi1 & phi2 for other AR(2) models
ARMAspec(model=list(ar=c(phi1,phi2)))

# ARMA(1,1)
win.graph(width=4.875, height=2.5,pointsize=8)
theta=0.8 ; phi=0.5 # Reset parameters for other ARMA(1,1) models
ARMAspec(model=list(ar=phi,ma=-theta))

# Seasonal AR
win.graph(width=4.875, height=2.5,pointsize=8)
phi=.5 ; PHI=.9 # Reset parameters for other models
ARMAspec(model=list(ar=phi,seasonal=list(sar=PHI,peroid=12)))

# Seasonal MA
win.graph(width=4.875, height=2.5,pointsize=8)
theta=.4 ; Theta=.9 # Reset parameters for other models
ARMAspec(model=list(ma=-theta,seasonal=list(sma=-Theta,period=12)))

# Sample spectral density for a simulated AR-1 process
win.graph(width=4.875,height=2.5,pointsize=8)
set.seed(271435)
n=200
phi=-0.6
y=arima.sim(model=list(ar=phi),n=n)
sp=spec(y,log='no',xlab='Frequency',ylab='Sample Spectral Density',main='',
sub='',demean=T)
lines(sp$freq,ARMAspec(model=list(ar=phi),freq=sp$freq,plot=F)$spec,
lty='dotted')
abline(h=0)

# Sample Spectral Density for AR(1) with phi=-0.6 and n = 200
win.graph(width=4.875,height=2.5,pointsize=8)
set.seed(271435); n=200; phi=-0.6
y=arima.sim(model=list(ar=phi),n=n)
k=kernel('daniell',m=5)
sp=spec(y,kernel=k,log='no',main='',sub='',
 xlab='Frequency',ylab='Sample Spectral Density')
lines(sp$freq,ARMAspec(model=list(ar=phi),freq=sp$freq,plot=F)$spec,
lty='dotted')

k=kernel('daniell',m=15)
sp=spec(y,kernel=k,log='no',main='',sub='',
 xlab='Frequency',ylab='Sample Spectral Density')
lines(sp$freq,ARMAspec(model=list(ar=phi),freq=sp$freq,plot=F)$spec,
lty='dotted')

# Sample Spectrum of White Noise
layout(1:3)
set.seed(1)
x<- rnorm(2048)
spectrum(x, log=c("no"))
spectrum(x, span=15, log=c("no"))
spectrum(x, span=65, log=c("no"))

# The Spectra in R's base package are scaled so that their areas are one-half the variance of the time series
# However, for a report there is no distinction between positive and negative frequencies, so it is usual
# to plot so that it is scaled to be equal to the variance.
layout(1:2)
width <- 2048
x.spec <- spectrum(x, span=65, log=c("no"))
spx <- x.spec$freq / width
spy <- 2* x.spec$spec
plot(spx, spy, xlab="Hz", ylab="variance/Hz", type="l")

# Sample Spectrum of AR(1) Positive Coefficient
set.seed(1)
x <- w <- rnorm(1024)
for (t in 2: 1024) x[t] <- .9*x[t-1] +w[t]
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x, log=c("no"))
spectrum(x,span=51, log=c("no"))

# Sample Spectrum of AR(1) Negative Coefficient
set.seed(1)
x <- w <- rnorm(1024)
for (t in 2: 1024) x[t] <- -.9*x[t-1] +w[t]
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x, log=c("no"))
spectrum(x,span=51, log=c("no"))

# Sample Spectrum of AR(2)
set.seed(1)
x <- w <- rnorm(1024)
for (t in 3: 1024) x[t] <- x[t-1]-.6*x[t-2] +w[t]
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x, log=c("no"))
spectrum(x,span=51, log=c("no"))

# Sample Spectrum of MA(1) Positive Coefficient
set.seed(1)
x <- w <- rnorm(1024)
for (t in 2: 1024) x[t] <- w[t]-.9*w[t-1]
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x,log=c("no"))
spectrum(x,span=51, log=c("no"))

# Sample Spectrum of MA(1) Negative Coefficient
set.seed(1)
x <- w <- rnorm(1024)
for (t in 2: 1024) x[t] <- w[t]+.9*w[t-1]
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x, log=c("no"))
spectrum(x,span=51, log=c("no"))

# Sample Spectrum of MA(2)
set.seed(1)
x <- w <- rnorm(1024)
for (t in 3: 1024) x[t] <- w[t]-w[t-1]+.6*w[t-2] +
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x, log=c("no"))
spectrum(x,span=51, log=c("no"))

# Sample Spectrum of AR(1) Monthly Seasonal Positive Coefficient
set.seed(1)
x <- w <- rnorm(1024)
for (t in 13: 1024) x[t] <- .9*x[t-12] +w[t]
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x, log=c("no"))
spectrum(x,span=5, log=c("no"))

# Sample Spectrum of MA(1) Monthly Seasonal Positive Coefficient
set.seed(1)
x <- w <- rnorm(1024)
for (t in 13: 1024) x[t] <- w[t]-.9*w[t-12]
layout(1:4)
plot(as.ts(x))
acf(x)
spectrum(x, log=c("no"))
spectrum(x,span=5, log=c("no"))

# Other spectral windows
win.graph(width=4.875,height=2,pointsize=10)
oldpar=par
par(mfrow=c(1,3))
plot(kernel("modified.daniell", c(3)),lwd=2,main='',ylim=c(0,.17),ylab=expression(W(k)))
abline(h=0)
plot(kernel("modified.daniell", c(3,3)),lwd=2,main='',ylim=c(0,.17),ylab=expression(W(k)))
abline(h=0)
plot(kernel("modified.daniell", c(3,3,3)),lwd=2,main='',ylim=c(0,.17),ylab=expression(W(k)))
abline(h=0)
par=oldpar

# Confidence Intervals for the Estimated Spectrum
# Note: Logging is the default, which we removed above.
win.graph(width=4.875,height=2.5,pointsize=8)
set.seed(271435); n=200; phi=-0.6
y=arima.sim(model=list(ar=phi),n=n)
k=kernel('daniell',m=15)
sp=spec(y,kernel=k,main='',sub='',xlab='Frequency',
ylab='Log(Smoothed Sample Spectrum)',ci.plot=T,ci.col=NULL)
lines(sp$freq,ARMAspec(model=list(ar=phi),sp$freq,plot=F)$spec,lty="dashed")
abline(h=0)

# Alternative approach to reporting the confidence interval.
sp=spec(y,span=31,sub='',xlab='Frequency',
ylab='Log(Smoothed Sample Spectrum)')
lines(sp$freq,ARMAspec(model=list(ar=phi),sp$freq,
plot=F)$spec,lty='dotted')
# Note that the crosshair can be visualized as sliding along the smoothed spectrum

# Non-Fourier frequencies and leakage
win.graph(width=4.875,height=2.5,pointsize=8)
t=1:96
f1=0.088; f2=14/96 # Note: 0.088 is NOT of the form m/96 for any integer m
y=3*cos(f1*2*pi*t)+sin(f2*2*pi*t)
spec(y,log='no',main='',sub='',type='h',lwd=2,
 ylab='Periodogram',xlab='Frequency')
abline(h=0)
axis(1,at=c(0.088,0.14583))
# Note that the lower peak at 14/96 is present. However, there is no peak at 0.088. Rather,
# There is leakage into the surrounding frequencies, exhibited by the higher peaks.

# Tapering is one method used to improve the issue with the leakage and sidelobes.
# Dirichlet kernel
win.graph(width=4.875,height=2.5,pointsize=8)
olfpar=par
par(mfrow=c(1,2))
n=100
f=seq(-.1,.1,.0001)
D=(1/n)*sin(pi*n*f)/sin(pi*f)
D[1001]=1
plot(x=f,y=D, typ='l',ylab='Dirichlet Kernel',xlab='Frequency',ylim=c(-.25,1.05))
abline(h=0)
# Tapered
Dt=filter(D,filter=c(.5,rep(0,100),1,rep(0,100),.5),method='convolution')
plot(x=f,y=Dt,typ='l',xlab='Frequency',ylim=c(-.25,1.05),ylab=expression(D[T]))
abline(h=0)
par=oldpar

# Tapering with the Cosine Bell
oldpar=par
par(mfrow=c(1,2))
t=1:100
y=rep(1,100)
tp=spec.taper(y,p=.5)
abline(h=0)
plot(t,tp,typ='l',ylab='Cosine Bell',xlab='Time')
abline(h=0)
taper=spec.taper(y,p=.1)
plot(t,taper,ylab='Split Cosine Bell',xlab='Time',typ='l')
abline(h=0)
par=oldpar


# Return to the star brightness data considered above
win.graph(width=4.875,height=7,pointsize=12)
opar=par()
par(mfrow=c(4,1),mar=c(2,1,1,1)+.1,oma=c(3,3,2,0))
temp=NULL
data(star)
for (taper in c(0,0.1,0.2,0.5) ) {
sp=spec(star,sub='',main='',taper=taper,plot=F)
temp=c(temp,sp$spec)
}
for (taper in c(0,0.1,0.2,0.5) ) {
sp=spec(star,sub='',main='',taper=taper,ylim=pmax(extendrange(temp),0.0000001),ci.col='black')
}
par(opar)

# Much of the preceding can also be done in TSA
# Simulated AR(1) with phi=-0.6 and n = 200 in TSA
win.graph(width=4.875,height=2.5,pointsize=8)
set.seed(271435)
n=200
phi=-0.6
y=arima.sim(model=list(ar=phi),n=n)
sp=spec(y,method='ar',main='',sub='',
 xlab='Frequency', ylab='Sample Spectral Density')
lines(sp$freq,ARMAspec(model=list(ar=phi),freq=sp$freq,plot=F)$spec,
lty='dotted')

# Spectral analysis of simulated series using TSA
win.graph(width=4.875,height=2.5,pointsize=8)
set.seed(271435)
n=100
phi1=1.5; phi2=-.75 # Reset parameter values to obtain Exhibits 14.9 & 14.10
y=arima.sim(model=list(ar=c(phi1,phi2)),n=n)
sp1=spec(y,main='',spans=3,sub='',lty='dotted', xlab='Frequency',
 ylab='Log(Estimated Spectral Density)')
sp2=spec(y,,spans=9,plot=F)
sp3=spec(y,,spans=15,plot=F)
lines(sp2$freq,sp2$spec,lty='dashed') # Use different colors to see the results more easily!
lines(sp3$freq,sp3$spec,lty='dotdash')
f=seq(0.001,.5,by=.001)
lines(f,ARMAspec(model=list(ar=c(phi1,phi2)),freq=f,plot=F)$spec,
lty='solid')

# AR Spectral Estimation : Estimated (dotted), True(solid)
sp4=spec(y,method='ar',main='',lty='dotted', xlab='Frequency',
 ylab='Log(Estimated AR Spectral Density)')
f=seq(0.001,.5,by=.001)
lines(f,ARMAspec(model=list(ar=c(phi1,phi2)),freq=f,plot=F)$spec,lty='solid')
sp4$method # This will tell you order of the AR model selected.


# Spectral Estimates for ARMA(1,1) process
win.graph(width=4.875,height=2.5,pointsize=8) # Exhibits 14.13 & 14.14
set.seed(324135)
n=500
phi=.5; theta=.8
y=arima.sim(model=list(ar=phi,ma=-theta),n=n)
sp1=spec(y,spans=11,main='',sub='',lty='dotted', xlab='Frequency',
 ylab='Log(Estimated Spectral Density)')
sp2=spec(y,spans=22,plot=F)
sp3=spec(y,spans=44,plot=F)
lines(sp2$freq,sp2$spec,lty='dashed')
lines(sp3$freq,sp3$spec,lty='dotdash')
f=seq(0.001,.5,by=.001)
lines(f,ARMAspec(model=list(ar=phi,ma=-theta),f,plot=F)$spec,lty='solid')

# AR Spectral Estimate for an ARMA(1,1) model
sp4=spec(y,method='ar',main='',lty='dotted',ylim=c(.15,1.9), xlab='Frequency',
 ylab='Log(Estimated AR Spectral Density)')
f=seq(0.001,.5,by=.001)
lines(f,ARMAspec(model=list(ar=phi,ma=-theta),f,plot=F)$spec,lty='solid')
sp4$method # This will tell you order of the AR model selected.

# Seasonal process
win.graph(width=4.875,height=2.5,pointsize=8)
set.seed(247135)
n=144
theta=.4;THETA=.9
y=arima.sim(model=list(ma=c(-theta,rep(0,10),-THETA,theta*THETA)),n=n)
sp1=spec(y,spans=6,main='',sub='',lty='dotted',ylim=c(.15,9),
 xlab='Frequency',ylab='Log(Estimated Spectral Density)')
sp2=spec(y,,spans=12,plot=F)
sp3=spec(y,,spans=24,plot=F)
lines(sp2$freq,sp2$spec,lty='dashed')
lines(sp3$freq,sp3$spec,lty='dotdash')
f=seq(0.001,.5,by=.001)
lines(f,ARMAspec(model=list(ma=-theta,seasonal=list(sma=-THETA,period=12)),
freq=f,plot=F)$spec,lty='solid')

# AR Spectral Estimates for a Seasonal Process
sp4=spec(y,method='ar',main='',ylim=c(.15,15),lty='dotted',
 xlab='Frequency',ylab='Log(Estimated AR Spectral Density)')
f=seq(0.001,.5,by=.001)
lines(f,ARMAspec(model=list(ma=-theta,seasonal=list(sma=-THETA,period=12)),
freq=f,plot=F)$spec,lty='solid')
sp4$method
